@@ -108,8 +108,6 @@ void CBasicFaxConversion::ConnectLog (cp_ncpi_all *pNCPI) {
             dassert (GetReceiveID() != 0);
             GetReceiveID()->Set (pNCPI->pReceiveID->GetData(), pNCPI->pReceiveID->GetLen());
             WriteLog (LOG_INFO, "\tStationID = %s\n" , GetReceiveID()->GetPointer());
-        } else {
-            GetReceiveID()->RemoveAll();
         }
         if (pNCPI->Rate) {
             m_BaudRate = pNCPI->Rate;
@@ -179,7 +177,7 @@ void CBasicFaxConversion::WriteXferLog (const char *cmd, tUInt commid, tUChar *j
     char *timestrf = "%D %H:%M";
     strftime (buf, sizeof (buf), timestrf, localtime (&m_StartTime));
     CDynamicString xferline (200);
-    xferline.Print ("%s\t%s\t%08u\t%s", buf, cmd, commid, devname);
+    xferline.Print ("%s\t%s\t%09u\t%s", buf, cmd, commid, devname);
     if (pJobid) {
         xferline.PrintAppend ("\t%u\t\"", *pJobid);
     } else {
@@ -266,6 +264,10 @@ tSInfo CBasicFaxConversion::OpenSend (CMultiString *pSendFileList, tUInt Resolut
             WriteLog (LOG_ERR, "Can't open TIF-File %s!\n", m_pCurSendFile->GetPointer());
             return iErr_File_NotExist;
         }
+        if (m_sTifFilter.CheckForCorrectFaxFile() == vFalse) {
+            WriteLog (LOG_ERR, "TIF-File %s doesn't look like a 1d G3 Fax!\n", m_pCurSendFile->GetPointer());
+            return iErr_File_OpenFailed;
+        }
         Resolution = (m_VResolution) ? m_VResolution : (m_sTifFilter.GetYRes()) ? m_sTifFilter.GetYRes() : 196;
         if (m_sSffFilter.Open ((Resolution >= 196) ? 1 : 0, 0) == vFalse) {
             WriteLog (LOG_ERR, "Can't allocate memory for data conversion to the SFF format!\n");
@@ -612,7 +614,7 @@ tSInfo CBasicFaxConversion::FillSffFilter (void) {
         case iWrn_Filter_PageEnd:
             m_PageCount++;
             m_sSffFilter.PutPageHeader (vFalse);
-            WriteLog (LOG_INFO, "Page %d was sended.\n", m_PageCount);
+            WriteLog (LOG_INFO, "Page %d was sent.\n", m_PageCount);
             break;
         case iWrn_Filter_LastPageEnd: {
             tBool isFinish = vTrue;
@@ -633,13 +635,18 @@ tSInfo CBasicFaxConversion::FillSffFilter (void) {
                         fret = iErr_File_NotExist;
                     }
                 } else if (m_sTifFilter.Open (m_pCurSendFile->GetPointer()) == vTrue) {
-                    WriteLog (LOG_INFO, "Page %d was sended.\n", m_PageCount);
-                    tUInt Resolution = (m_VResolution) ? m_VResolution :
-                                    (m_sTifFilter.GetYRes()) ? m_sTifFilter.GetYRes() : 196;
-                    m_sSffFilter.SetVRes ((Resolution >= 196) ? 1 : 0);
-                    WriteLog (LOG_INFO, "Start transfer of TIFF-File %s with %s resolution.\n",
-                              m_pCurSendFile->GetPointer(), (Resolution >= 196) ? "fine" : "normal");
-                    isFinish = vFalse;
+                    if (m_sTifFilter.CheckForCorrectFaxFile() == vFalse) {
+                        WriteLog (LOG_ERR, "TIF-File %s doesn't look like a G3 Fax!\n", m_pCurSendFile->GetPointer());
+                        fret = iErr_File_OpenFailed;
+                    } else {
+                        WriteLog (LOG_INFO, "Page %d was sent.\n", m_PageCount);
+                        tUInt Resolution = (m_VResolution) ? m_VResolution :
+                                        (m_sTifFilter.GetYRes()) ? m_sTifFilter.GetYRes() : 196;
+                        m_sSffFilter.SetVRes ((Resolution >= 196) ? 1 : 0);
+                        WriteLog (LOG_INFO, "Start transfer of TIFF-File %s with %s resolution.\n",
+                                m_pCurSendFile->GetPointer(), (Resolution >= 196) ? "fine" : "normal");
+                        isFinish = vFalse;
+                    }
                 } else {
                     WriteLog (LOG_ERR, "Can't open TIFF-File %s.\n", m_pCurSendFile->GetPointer());
                     fret = iErr_File_NotExist;
@@ -650,7 +657,7 @@ tSInfo CBasicFaxConversion::FillSffFilter (void) {
             dprint ("i_LastPageEnd-Recv (isFinish=%x) ", isFinish);
             if (isFinish == vTrue) {
                 if (m_format != FaxFormat_G3) {
-                    WriteLog (LOG_INFO, "Page %d was sended. - Last Page!\n", m_PageCount);
+                    WriteLog (LOG_INFO, "Page %d was sent. - Last Page!\n", m_PageCount);
                 }
                 m_finishedFaxFile++;
                 return fret;
