@@ -86,7 +86,7 @@ CFaxReceive::~CFaxReceive (void) {
 
 tBool CFaxReceive::StartReceive (void) {
     dhead ("CFaxReceive::StartReceive", DCON_CFaxReceive);
-    cp_b3configuration_faxext B3Config (1, &FaxDevice->FaxNumber); //Identifier.GetPointer();
+    cp_b3configuration_faxext B3Config (b3_faxextopt_highres | b3_faxextopt_jbig, &FaxDevice->FaxNumber);
     SetB1Parameters (b1prot_G3Fax);
     SetB2Parameters (b2prot_G3Fax);
     SetB3Parameters (b3prot_G3Fax_extended, &B3Config);
@@ -163,7 +163,7 @@ void CFaxReceive::IncomingCall (c_cipvalue CIPValue, CConstString *pOppositeNumb
             WriteLog (LOG_ERR, "Can't create log file for incomming call!\n");
             // TODO: better disconnect the call?
         }
-        WriteLog (LOG_INFO, "SESSION BEGIN %08u %s\n", jobNr, FaxDevice->FaxNumber.GetPointer());
+        WriteLog (LOG_INFO, "SESSION BEGIN %09u %s\n", jobNr, FaxDevice->FaxNumber.GetPointer());
     }
     if (InfoText.ErrorDetected()) {
         WriteLog (LOG_ERR, "Memory Full during generation of Log output!\n");
@@ -255,9 +255,12 @@ void CFaxReceive::GetData (tUByte *Data, tUShort DataLength, tUShort DataHandle)
 
 void CFaxReceive::DoDisconnecting (c_info, cp_ncpi_all *pNCPI) {
     dhead ("CFaxReceive::DoDisconnecting", DCON_CFaxReceive);
+    dwarning (pNCPI != 0);
     StopRecv();
     if (pNCPI) {
-        if (GetPageCount() != pNCPI->Pages) {
+        if (GetFormat() == FaxFormat_SFF) {
+            SetPageCount (pNCPI->Pages);
+        } else if (GetPageCount() != pNCPI->Pages) {
             WriteLog (LOG_ERR, "Discrepance between CAPI's (%d) and internal (%d) number of pages!\n",
                       pNCPI->Pages, GetPageCount());
             //SetPageCount (pNCPI->Pages);
@@ -280,6 +283,26 @@ void CFaxReceive::IsDisconnected (c_info Reason) {
     #endif
 
     if (GetRecvFiles() && (GetRecvFiles()->IsEmpty() == vFalse)) {
+        tUInt CountIllegalLines = 0;
+        tUInt CountAllLines = 0;
+        if ((GetFormat() != FaxFormat_SFF) && (GetRecvIllegalLines (&CountIllegalLines, &CountAllLines))) {
+            if (!CountAllLines) {
+                WriteLog (LOG_ERR, "No fax data was received!\n");
+            } else {
+                tUInt percent = CountIllegalLines * 100 / CountAllLines;
+                if (percent > 10) {
+                    WriteLog (LOG_ERR, "Around %d%% of the received fax is corrupt (%d lines)!\n", percent,
+                                       CountIllegalLines);
+                } else {
+                    WriteLog (LOG_INFO, "Received some illegal lines (%d lines = %d%% of fax),\n", CountIllegalLines,
+                                        percent);
+                }
+            }
+        }
+        if (GetReceiveID()->IsEmpty()) {
+            dwarning (0);
+            GetReceiveID()->Set ("-");
+        }
         if (FaxDevice->MainClass->FaxRcvdCmd.IsEmpty() == vFalse) {
             CDynamicString executeCommand;
             executeCommand.Set (&FaxDevice->MainClass->FaxRcvdCmd);
@@ -291,18 +314,24 @@ void CFaxReceive::IsDisconnected (c_info Reason) {
                 case 0:
                     StateText = "Can't build up connection for incoming call.";
                     break;
+                case 3:
+                    if (CountAllLines != 0) {
+                        if (CountIllegalLines * 100 / CountAllLines > 50) {
+                            StateText = "More than half of the fax is corrupted!";
+                        }
+                        break;
+                    }
+                    // no break!
                 case 1:
                     StateText = "Connection was established but no fax data was received.";
                     break;
                 case 2:
                     StateText = "Connection droped unexpected during fax receiving.";
                     break;
-                case 3:
-                    break;
                 }
 
                 // Hylafax: <qfile> <ModemDeviceID> <CommID> <Reason> <CIDNumber> <CIDName> <destination>
-                executeCommand.PrintAppend (" \"%S\" \"%S\" \"%08u\" \"%s\" \"%S\" \"\" \"%S\"",
+                executeCommand.PrintAppend (" \"%S\" \"%S\" \"%09u\" \"%s\" \"%S\" \"\" \"%S\"",
                                             GetRecvFiles()->GetFirst(), &FaxDevice->DeviceName, jobNr,
                                             StateText, GetReceiveID(), &MyNumber);
                 WriteXferLog ("RECV", jobNr, 0, 0, (char *)FaxDevice->DeviceName.GetPointer(), 0,
@@ -315,14 +344,20 @@ void CFaxReceive::IsDisconnected (c_info Reason) {
                 case 0:
                     recvStatus = 6;
                     break;
+                case 3:
+                    if (CountAllLines != 0) {
+                        if (CountIllegalLines * 100 / CountAllLines > 50) {
+                            recvStatus = 12;
+                        }
+                        break;
+                    }
+                    // no break!
                 case 1:
                     recvStatus = 7;
                     break;
                 case 2:
                     recvStatus = 12;
                     break;
-                case 3:
-                    break;
                 }
 
                 executeCommand.PrintAppend (" %u 0x%X \"", recvStatus, Reason);
@@ -448,7 +483,7 @@ tBool CFaxReceive::NotifyFIFO (char Type) {
     // Modem part
     case 'I':
         if (jobNr > 0) {
-            FaxInfo.Print ("%08u", jobNr);
+            FaxInfo.Print ("%09u", jobNr);
             text = (char *)FaxInfo.GetPointer();
         }
         break;
@@ -463,7 +498,7 @@ tBool CFaxReceive::NotifyFIFO (char Type) {
     case 'D':
         if (GetRecvFiles() && (GetRecvFiles()->IsEmpty() == vFalse)) {
             dassert (GetRecvFiles()->GetFirst()->IsEmpty() == vFalse);
-            FaxInfo. Print ("%lx,%x,%x,%S,%08u,\"%S\",\"%S\",\"%s\"", (tUInt64)(time (0) - GetStartTime()),
+            FaxInfo. Print ("%lx,%x,%x,%S,%09u,\"%S\",\"%S\",\"%s\"", (tUInt64)(time (0) - GetStartTime()),
                             GetPageCount(), FaxPropEncode (GetResolution(), GetBaudRate()),
                             GetRecvFiles()->GetFirst(), jobNr, GetReceiveID(), &ONumber, "");
             text = (char *)FaxInfo.GetPointer();
@@ -505,7 +540,7 @@ tBool CFaxReceive::GetJobFile (void) {
         if (tmpJobFileNr < 1 || tmpJobFileNr > MAX_SEQ_NUMBER) {
             tmpJobFileNr = 1;
         }
-        sprintf (astring, HYLAFAX_LOG_DIR "c%08u", tmpJobFileNr);
+        sprintf (astring, HYLAFAX_LOG_DIR "c%09u", tmpJobFileNr);
         tmpFileD = open (astring, O_RDWR | O_CREAT | O_EXCL, FaxDevice->MainClass->LogFileMode);
         dprint ("try(%s,%u,%d) ", astring, strlen (astring), tmpFileD);
         if (tmpFileD >= 0) {
