@@ -61,6 +61,7 @@ CFaxSend::CFaxSend (void (*FinishedFunc)(void))
     finishFunc (FinishedFunc),
     SpoolDir (DEFAULT_SPOOL_DIRECTORY),
     PollRcvdCmd (HYLAFAX_POLL_CMD),
+    LogFileName (DEFAULT_C4H_LOGFILENAME),
     LogLevel (0),
     LogFileMode (DEFAULT_FILE_MODE),
     SuppressMSN (vFalse),
@@ -94,19 +95,46 @@ CFaxSend::~CFaxSend (void) {
 /*===========================================================================*\
 \*===========================================================================*/
 
-tBool CFaxSend::Config (char *confName, char *devName, FaxFormatType faxType, int argc, char **argv) {
+tBool CFaxSend::Config (tChar *confName, char *devName, FaxFormatType faxType, int argc, char **argv) {
     dhead ("CFaxSend::Config", DCON_CFaxSend);
     dparams ("%s,%x,%x", confName, faxType, argc);
     dassert (confName != 0);
     dassert (devName != 0);
     dassert (argv != 0);
-    tUInt RingingTimeVal = 0;
-    tUInt OutController  = 1;
     DeviceName.Set (devName);
     SetFormat (faxType);
     optind = 1;
-    CConfigFileParser FileParser;
-    switch (FileParser.ParseFile ((tUChar *)confName)) {
+
+    tUInt                 RingingTimeVal = 0;
+    tUInt                 OutController  = 1;
+    CConfigFileParser     FileParser;
+    CConfigParserSection *mainSect;
+
+    ConfigAssignment_Item MainAssignments[] = {
+        { "SpoolDir",       ConfigAssignment_String, &SpoolDir },
+        { "PollRcvdCmd",    ConfigAssignment_String, &PollRcvdCmd },
+        { "LogFile",        ConfigAssignment_String, &LogFileName },
+        { "LogFileMode",    ConfigAssignment_UInt,   &LogFileMode },
+        { "LogTraceLevel",  ConfigAssignment_UInt,   &LogLevel },
+        { 0,                ConfigAssignment_UInt,   0 }
+    };
+    ConfigAssignment_Item Assignments[] = {
+        { "HylafaxDeviceName",  ConfigAssignment_String, &DeviceName },
+        { "FAXNumber",          ConfigAssignment_String, &OrgFaxNumber },
+        { "LocalIdentifier",    ConfigAssignment_String, &OrgIdentifier },
+        { "OutgoingMSN",        ConfigAssignment_String, &OutMSN },
+        { "NumberPrefix",       ConfigAssignment_String, &NumberPrefix },
+        { "NumberPlusReplacer", ConfigAssignment_String, &NumberPlusReplace },
+        { "OutgoingController", ConfigAssignment_UInt,   &OutController },
+        { "SuppressMSN",        ConfigAssignment_UInt,   &SuppressMSN },
+        { "UseISDNFaxService",  ConfigAssignment_UInt,   &UseFaxCIPValue },
+        { "RingingDuration",    ConfigAssignment_UInt,   &RingingTimeVal },
+        { "RecvFileMode",       ConfigAssignment_UInt,   &RecvFileMode },
+        { 0,                    ConfigAssignment_UInt,   0 }
+    };
+
+    printf ("The config file is %s.\n", confName);
+    switch (FileParser.ParseFile (confName)) {
     case iErr_File_OpenFailed:
         printf ("Can't open config file \"%s\". Use default values.\n", confName);
         break;
@@ -121,45 +149,25 @@ tBool CFaxSend::Config (char *confName, char *devName, FaxFormatType faxType, in
         printf ("Can't read config file \"%s\"!\n", confName);
         return vFalse;
     }
-    ConfigAssignment_Item MainAssignments[] = {
-        { "SpoolDir",       ConfigAssignment_String, &SpoolDir },
-        { "PollRcvdCmd",    ConfigAssignment_String, &PollRcvdCmd },
-        { "LogFile",        ConfigAssignment_String, &LogFileName },
-        { "LogFileMode",    ConfigAssignment_UInt,   &LogFileMode },
-        { "LogTraceLevel",  ConfigAssignment_UInt,   &LogLevel },
-        { 0,                ConfigAssignment_UInt,   0 }
-    };
-    ConfigAssignment_Item Assignments[] = {
-        { "HylafaxDeviceName",  ConfigAssignment_String, &DeviceName },
-        { "FAXNumber",          ConfigAssignment_String, &FaxNumber },
-        { "LocalIdentifier",    ConfigAssignment_String, &Identifier },
-        { "OutgoingMSN",        ConfigAssignment_String, &OutMSN },
-        { "NumberPrefix",       ConfigAssignment_String, &NumberPrefix },
-        { "NumberPlusReplacer", ConfigAssignment_String, &NumberPlusReplace },
-        { "OutgoingController", ConfigAssignment_UInt,   &OutController },
-        { "SuppressMSN",        ConfigAssignment_UInt,   &SuppressMSN },
-        { "UseISDNFaxService",  ConfigAssignment_UInt,   &UseFaxCIPValue },
-        { "RingingDuration",    ConfigAssignment_UInt,   &RingingTimeVal },
-        { "RecvFileMode",       ConfigAssignment_UInt,   &RecvFileMode },
-        { 0,                    ConfigAssignment_UInt,   0 }
-    };
-    CConfigParserSection *mainSect = FileParser.GetMainSection();
-    if (mainSect) {
-        ConfigValuesAssignment (&mainSect->ValueList, &MainAssignments[0]);
-        for (CConfigParserSection *laufSect = (CConfigParserSection *)mainSect->SectionList.GetFirst();
-             laufSect; laufSect = (CConfigParserSection *)laufSect->GetNext()) {
-
-            if (ConfigValuesTestName (&laufSect->ValueList, &Assignments[0]) != 1) {
-                ConfigValuesAssignment (&laufSect->ValueList, &Assignments[0]);
-                SetOutController (OutController);
-            }
-        }
-        mainSect->ValueList.RemoveAndDeleteAll();
-        mainSect->SectionList.RemoveAndDeleteAll();
+    mainSect = FileParser.GetMainSection();
+    if (!mainSect) {
+        printf ("Not enough Memory for config file parsing!\n");
+        return vFalse;
     }
+    ConfigValuesAssignment (&mainSect->ValueList, &MainAssignments[0]);
+    for (CConfigParserSection *laufSect = (CConfigParserSection *)mainSect->SectionList.GetFirst();
+        laufSect; laufSect = (CConfigParserSection *)laufSect->GetNext()) {
+
+        if (ConfigValuesTestName (&laufSect->ValueList, &Assignments[0]) != 1) {
+            ConfigValuesAssignment (&laufSect->ValueList, &Assignments[0]);
+            SetOutController (OutController);
+        }
+    }
+    mainSect->ValueList.RemoveAndDeleteAll();
+    mainSect->SectionList.RemoveAndDeleteAll();
 
     char ch;
-    while ((ch = getopt (argc, argv, "Hh?vlpnf:C:c:d:m:P:")) != -1) {
+    while ((ch = getopt (argc, argv, "Hh?vLlpnf:C:c:d:m:P:")) != -1) {
         switch (ch) {
         case 'c':
             if (mainSect && optarg && *optarg) {
@@ -176,24 +184,11 @@ tBool CFaxSend::Config (char *confName, char *devName, FaxFormatType faxType, in
         case 'v':
             enableEcho = vTrue;
             break;
+        case 'L':
+            LogLevel = LOG_TRACE_ALL;
+            break;
         }
     }
-
-    SetRingingTimeout (RingingTimeVal);
-    return vTrue;
-}
-
-/*===========================================================================*\
-\*===========================================================================*/
-
-tSInfo CFaxSend::Send (CDynamicString *Number, CMultiString *FaxFilesList, tUInt Resolution,
-                       tUInt jobID, tUInt *pCommID, CDynamicString *pPollStr, CDynamicString *pTagLineFormat) {
-    dhead ("CFaxSend::Send", DCON_CFaxSend);
-    dassert (Number != 0);
-    dparams ("%s,%u,%u", Number->GetPointer(), Resolution, jobID);
-    dassert (FaxFilesList != 0);
-    dassert (pCommID != 0);
-    dassert (pTagLineFormat != 0);
     if (  (LogFileName.IsEmpty() == vFalse) && LogLevel) {
         hLogFile = fopen ((char *)LogFileName.GetPointer(), "a");
         dprint ("LogFile=%x-%s ", hLogFile, (char *)LogFileName.GetPointer());
@@ -201,11 +196,32 @@ tSInfo CFaxSend::Send (CDynamicString *Number, CMultiString *FaxFilesList, tUInt
             fchmod (fileno (hLogFile), LogFileMode);
         }
     }
+    WriteLog (LOG_INFO, "\nC4H-Version " VERSION
+    #ifdef C_PLATFORM_64
+                " (64Bit).\n\n");
+    #else
+                " (32Bit).\n\n");
+    #endif
+    Identifier.Set (&OrgIdentifier);
+    FaxNumber.Set (&OrgFaxNumber);
+    SetRingingTimeout (RingingTimeVal);
+    if (GetFormat() == FaxFormat_Hylafax) {
+        SpoolDir.SetRelative (HYLAFAX_POLL_DIR); // Adjust SpoolDir to correct place
+    }
+    return vTrue;
+}
+
+/*===========================================================================*\
+\*===========================================================================*/
+
+void CFaxSend::SetTagLine (CDynamicString *pTagLineFormat) {
+    dhead ("CFaxSend::SetTagLine", DCON_CFaxSend);
+    dassert (pTagLineFormat != 0);
     // parsing TagLine
     if (!pTagLineFormat->IsEmpty()) {
         tUInt pos = pTagLineFormat->FindChar ('|');
-        if (pos != MAXVAL_tSize) {
-            // '|' was found! 
+        if (pos != MAXVAL_tUInt) {
+            // '|' was found!
             if (pos > 0) {
                 // Ignore empty Identifier
                 Identifier.Set (pTagLineFormat->GetPointer(), pos);
@@ -218,7 +234,22 @@ tSInfo CFaxSend::Send (CDynamicString *Number, CMultiString *FaxFilesList, tUInt
         } else {
             Identifier.Set (pTagLineFormat);
         }
+    } else {
+        Identifier.Set (&OrgIdentifier);
+        FaxNumber.Set (&OrgFaxNumber);
     }
+}
+
+/*===========================================================================*\
+\*===========================================================================*/
+
+tSInfo CFaxSend::Send (CDynamicString *Number, CMultiString *FaxFilesList, tUInt Resolution,
+                       tUInt jobID, tUInt *pCommID, CDynamicString *pPollStr) {
+    dhead ("CFaxSend::Send", DCON_CFaxSend);
+    dassert (Number != 0);
+    dparams ("%s,%u,%u", Number->GetPointer(), Resolution, jobID);
+    dassert (FaxFilesList != 0);
+    exitState = 0;
     // check params for fax polling
     PollString.RemoveAll();
     if (pPollStr && (pPollStr->IsEmpty() == vFalse)) {
@@ -247,14 +278,18 @@ tSInfo CFaxSend::Send (CDynamicString *Number, CMultiString *FaxFilesList, tUInt
 
     char *ModeText;
     switch (GetFormat()) {
-    default:
-        SpoolDir.SetRelative (HYLAFAX_POLL_DIR); // Adjust SpoolDir to correct place
+    default: {
         ModeText = "Hylafax";
-        *pCommID = GetJobFile();
-        WriteLog (LOG_INFO, "SESSION BEGIN %08u %s\n", jobID, FaxNumber.GetPointer());
-        WriteLog (LOG_INFO, "%s FAX: JOB %u DEST %s COMMID %08u\n",
-                  PollString.IsEmpty() ? "SEND" : "POLL", jobID, Number->GetPointer(), *pCommID);
+        tUInt commID = GetJobFile();
+        dassert (pCommID != 0);
+        if (*pCommID) {
+            *pCommID = commID;
+        }
+        WriteLog (LOG_INFO, "SESSION BEGIN %09u %s\n", jobID, FaxNumber.GetPointer());
+        WriteLog (LOG_INFO, "%s FAX: JOB %u DEST %s COMMID %09u\n",
+                  PollString.IsEmpty() ? "SEND" : "POLL", jobID, Number->GetPointer(), commID);
         break;
+    }
     case FaxFormat_TIFF:
         ModeText = "TIFF";
         break;
@@ -267,7 +302,7 @@ tSInfo CFaxSend::Send (CDynamicString *Number, CMultiString *FaxFilesList, tUInt
     }
     TelNumber.ErrorDetected();
     TelNumber.Set (&NumberPrefix);
-    if (  (Number->GetChar (0) == '+') 
+    if (  (Number->GetChar (0) == '+')
        && (!NumberPlusReplace.IsEmpty())) {
         if (Number->GetLen() <= 1) {
             WriteLog (LOG_ERR, "Telephonenumber to small (only includes '+').\n");
@@ -286,16 +321,16 @@ tSInfo CFaxSend::Send (CDynamicString *Number, CMultiString *FaxFilesList, tUInt
     }
 
     tSInfo fret = i_Okay;
-    cp_b3configuration_faxext B3Config (1, &FaxNumber, &Identifier);
-    cp_ncpi_faxext            NCPI     (1);
+    cp_b3configuration_faxext B3Config (b3_faxextopt_highres | b3_faxextopt_jbig, &FaxNumber, &Identifier);
+    cp_ncpi_faxext            NCPI     (ncpi_faxextopt_highres);
 
-    WriteLog (LOG_INFO, "Try to connect to fax number %s in %s mode%s on controller %d.\n", 
+    WriteLog (LOG_INFO, "Try to connect to fax number %s in %s mode%s on controller %d.\n",
               TelNumber.GetPointer(), ModeText, PollString.IsEmpty() ? "" : " (polling active)",
               GetOutController());
     if (PollString.IsEmpty()) {
         fret = OpenSend (FaxFilesList, Resolution);
     } else {
-        NCPI.Options |= 2;  // Enable FaxPolling
+        NCPI.Options |= ncpi_faxextopt_poll;  // Enable FaxPolling
     }
     if (infoError (fret)) {
         switch (fret) {
@@ -534,7 +569,7 @@ void CFaxSend::IsDisconnected (c_info Reason) {
             }
 
             CDynamicString commStr;
-            commStr.Print ("%08u", m_commID);
+            commStr.Print ("%09u", m_commID);
 
             // Hylafax: <mailaddr> <qfile> <ModemDeviceID> <CommID> <Reason>
             executeCommand.PrintAppend (" \"%S\" \"%S\" \"%S\" \"%S\" \"%s\"", &PollString,
@@ -746,7 +781,7 @@ tUInt CFaxSend::GetJobFile (void) {
         if (tmpJobFileNr < 1 || tmpJobFileNr > MAX_SEQ_NUMBER) {
             tmpJobFileNr = 1;
         }
-        sprintf (astring, HYLAFAX_LOG_DIR "c%08u", tmpJobFileNr);
+        sprintf (astring, HYLAFAX_LOG_DIR "c%09u", tmpJobFileNr);
         tmpFileD = open (astring, O_RDWR | O_CREAT | O_EXCL, LogFileMode);
         dprint ("try(%s,%u,%d) ", astring, strlen (astring), tmpFileD);
         if (tmpFileD >= 0) {
@@ -786,6 +821,7 @@ void CFaxSend::CloseJobFile (void) {
         fclose (hJobFile);
         hJobFile = 0;
     }
+    m_commID = 0;
 }
 
 
