@@ -21,6 +21,7 @@
 #include <semaphore.h>
 #include <unistd.h>
 #include <syslog.h>
+#include <errno.h>
 #include "faxsend.h"
 #include "ConfAssi.h"
 #include "FilePars.h"
@@ -39,33 +40,32 @@ static sem_t syncsema;
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
 
-void FinishFunc (void) {
+static void FinishFunc (void) {
     sem_post (&syncsema);
 }
 
 /*===========================================================================*\
 \*===========================================================================*/
 
-void WriteQFile (char *QFileName, tUInt commID, CFaxSend *fs, char **statustext) {
+static void WriteQFile (tChar *QFileName, tUInt commID, tUInt HylafaxExitState, CFaxSend *fs, char **statustext) {
     dhead ("WriteQFile", DCON_CFaxSend);
     dassert (QFileName != 0);
     CDynamicQueue textQ (60, sizeof (char *));
     char  text[4 * MAX_STRING_SIZE];
     char *ne;
-    tUInt dialortry  = 1;
     tSInt triescount;
-
-    FILE *qf         = fopen (QFileName, "r+");
-    
-    *statustext = "";
+    tUInt dialortry  = 1;
+    *statustext      = "";
+    FILE *qf         = fopen ((char *)QFileName, "r+");
 
     if (!qf) {
         return;
     }
     switch (fs->exitState) {
     case 255:
-	*statustext = "Sending job";
-	break;
+        *statustext = "Sending job";
+        dialortry  = 0;
+        break;
     case 0:
         dialortry  = 2;
         break;
@@ -126,21 +126,24 @@ void WriteQFile (char *QFileName, tUInt commID, CFaxSend *fs, char **statustext)
             continue;
         }
         *ne = '\0';
-        if (strcasecmp ("status", text) == 0) {
+        if (strcasecmp ("returned", text) == 0) {
+            dprint ("found returned ");
+            sprintf (ne + 1, "%u\n", HylafaxExitState);
+        } else if (strcasecmp ("status", text) == 0) {
             dprint ("found status ");
             sprintf (ne + 1, "%s\n", *statustext);
         } else if (commID && (strcasecmp ("commid", text) == 0)) {
             dprint ("found commid ");
-            sprintf (ne + 1, "%08u\n", commID);
+            sprintf (ne + 1, "%09u\n", commID);
         } else if (strcasecmp ("npages", text) == 0) {
             dprint ("found npages ");
             sprintf (ne + 1, "%u\n", fs->GetPageCount());
-        } if ((strcasecmp ("ndials", text) == 0) || (strcasecmp ("totdials", text) == 0)) {
+        } else if ((dialortry == 1) && ((strcasecmp ("ndials", text) == 0) || (strcasecmp ("totdials", text) == 0))) {
             dprint ("found _dials ");
             triescount = atol (ne + 1) + 1;
             if (triescount <= 0) {
                 dprintErr ("negative Value ");
-                triescount = 2;
+                triescount = 1;
             }
             sprintf (ne + 1, "%u\n", triescount);
         } else if ((dialortry == 2) && ((strcasecmp ("ntries", text) == 0) || (strcasecmp ("tottries", text) == 0))) {
@@ -148,7 +151,7 @@ void WriteQFile (char *QFileName, tUInt commID, CFaxSend *fs, char **statustext)
             triescount = atol (ne + 1) + 1;
             if (triescount <= 0) {
                 dprintErr ("negative Value ");
-                triescount = 2;
+                triescount = 1;
             }
             sprintf (ne + 1, "%u\n", triescount);
         } else if (strcasecmp ("signalrate", text) == 0) {
@@ -181,21 +184,200 @@ void WriteQFile (char *QFileName, tUInt commID, CFaxSend *fs, char **statustext)
 /*===========================================================================*\
 \*===========================================================================*/
 
-void usage (char *prgName) {
+static void OneSendSession (CFaxSend *pFS, tUInt Resolution, tUInt JobID, tUInt *pCommID, CDynamicString *pTelNumber,
+                            CDynamicString *pPollStr, CMultiString *pFileList) {
+    dhead ("OneSendSession", DCON_CFaxSend);
+    dassert (pFS != 0);
+    dassert (pTelNumber != 0);
+    dassert (pPollStr != 0);
+    dassert (pFileList != 0);
+    if ((pFileList->IsEmpty() == vTrue) && (pPollStr->IsEmpty() == vTrue)) {
+        pFS->WriteLog (LOG_ERR, "No file to send specified! Forgot the format option (-f)?\n\n");
+        pFS->exitState = 13;
+        return;
+    }
+    if (pTelNumber->IsEmpty() == vFalse) {
+        pTelNumber->SetLen (s_strkeep (pTelNumber->GetPointer(), (tChar *)"+0123456789*#RrMm"));
+    }
+    if (pTelNumber->IsEmpty() == vTrue) {
+        pFS->WriteLog (LOG_ERR, "No telephone number specified!\n\n");
+        pFS->exitState = 13;
+        return;
+    }
+    if (pFS->Send (pTelNumber,
+                   pFileList,
+                   Resolution,
+                   JobID,
+                   pCommID,
+                   pPollStr) == i_Okay) {
+        while (sem_wait (&syncsema)) {
+            if (errno != EINTR) {
+                break;
+            }
+        }
+    }
+    dprint ("fs.exitState=%x ", pFS->exitState);
+    pFS->CloseJobFile();
+}
+
+
+/*===========================================================================*\
+\*===========================================================================*/
+
+static void OneQFileSend (CFaxSend *pFS, char *deviceName, CDynamicString *pQFile) {
+    dhead ("OneQFileSend", DCON_CFaxSend);
+    dassert (pFS != 0);
+    dassert (deviceName != 0);
+    dassert (pQFile != 0);
+    dparams ("%s", pQFile->GetPointer());
+    CMultiString      FaxFileList;
+    CDynamicString    TelNumber;
+    CDynamicString    JobTag;
+    CDynamicString    MailAddr;
+    CDynamicString    PollStr;
+    CDynamicString    TagLineFormat;
+    tUInt             Resolution    = 0;
+    tUInt             JobID         = 0;
+    tUInt             commID        = 0;
+    tUInt             HylaExitState = 1;
+    char             *statustext;
+    CConfigFileParser FileParser;
+    switch (FileParser.ParseFile (pQFile->GetPointer())) {
+    case iErr_File_OpenFailed:
+        pFS->WriteLog (LOG_ERR, "Can't open qfile \"%s\".\n", (char *)pQFile->GetPointer());
+        pFS->exitState = 3;
+        return;
+    case i_Okay:
+        break;
+    case iErr_OutOfMemory:
+        dassert (0);
+        pFS->WriteLog (LOG_ERR, "Running out of memory during reading of qfile!\n");
+        pFS->exitState = 3;
+        return;
+    default:
+        dassert (0);
+        pFS->WriteLog (LOG_ERR, "Can't read qfile \"%s\"!\n", (char *)pQFile->GetPointer());
+        pFS->exitState = 3;
+        return;
+    }
+    CConfigParserSection *mainSect = FileParser.GetMainSection();
+    if (!mainSect) {
+        pFS->WriteLog (LOG_ERR, "Can't find needed information in qfile \"%s\".\n", (char *)pQFile->GetPointer());
+        pFS->exitState = 3;
+        return;
+    }
+    ConfigAssignment_Item Assignments[] = {
+        { "number",     ConfigAssignment_String,     &TelNumber },
+        { "fax",        ConfigAssignment_StringList, &FaxFileList },
+        { "resolution", ConfigAssignment_UInt,       &Resolution },
+        { "jobid",      ConfigAssignment_UInt,       &JobID },
+        { "jobtag",     ConfigAssignment_String,     &JobTag },
+        { "mailaddr",   ConfigAssignment_String,     &MailAddr },
+        { "poll",       ConfigAssignment_String,     &PollStr },
+        { "tagline",    ConfigAssignment_String,     &TagLineFormat },
+        { 0,            ConfigAssignment_UInt,       0 }
+    };
+    ConfigValuesAssignment (&mainSect->ValueList, &Assignments[0]);
+
+    // for polling we need the mailaddr
+    if (PollStr.IsEmpty() == vFalse) {
+        PollStr.Set (&MailAddr);
+        if (PollStr.IsEmpty() == vTrue) {
+            PollStr.Set (" ", 1);
+        }
+    }
+
+    // remove numbers from fax list
+    COneMultiString *pLauf = FaxFileList.GetFirst();
+    while (pLauf != 0) {
+        dassert (pLauf->GetLen() > 0);
+        switch (pLauf->GetChar (0)) {
+        case '0':
+        case '1':
+        case '2':
+        case '3':
+        case '4':
+        case '5':
+        case '6':
+        case '7':
+        case '8':
+        case '9':
+            {
+                COneMultiString *toDel = pLauf;
+                pLauf = pLauf->GetNext();
+                delete toDel;
+            }
+            break;
+        default:
+            pLauf = pLauf->GetNext();
+            break;
+        }
+    }
+    // Write "Sending job" into queue file
+    pFS->exitState = 255;
+    WriteQFile (pQFile->GetPointer(), commID, 0, pFS, &statustext);
+    pFS->exitState = 0;
+    pFS->SetTagLine (&TagLineFormat);
+
+    OneSendSession (pFS, Resolution, JobID, &commID, &TelNumber, &PollStr, &FaxFileList);
+    dprint ("fs.exitState=%x ", pFS->exitState);
+
+    switch (pFS->exitState) {
+    case 2:
+    case 4:
+    case 5:
+    case 6:
+    case 7:
+    case 10:
+    case 12:
+        // retry
+        HylaExitState = 0;
+        break;
+    case 15:
+    default:
+        // error
+        HylaExitState = 1;
+        break;
+    case 0:
+        // okay
+        HylaExitState = 2;
+        break;
+    case 1:
+    case 3:
+    case 11:
+        // reformat
+        HylaExitState = 3;
+        break;
+    }
+    WriteQFile (pQFile->GetPointer(), commID, HylaExitState, pFS, &statustext);
+    pFS->exitState = HylaExitState;
+    pFS->WriteXferLog (PollStr.IsEmpty() ? "SEND" : "POLL", commID, JobTag.GetPointer(), &JobID,
+                       deviceName, &MailAddr, &TelNumber, statustext);
+    dprint ("fs.exitState=%x ", pFS->exitState);
+    pFS->WriteLog (LOG_INFO, "SESSION END\n");
+    pFS->CloseJobFile();
+}
+
+
+/*===========================================================================*\
+\*===========================================================================*/
+
+static void usage (char *prgName) {
     char *name = strrchr (prgName, '/');
     if (!name) {
         name = prgName;
     } else {
         name++;
     }
-    printf ("usage: %s [-f format] [-C cfile] [-c param] [-m dev]\n\t\t"
+    printf ("usage: %s [-f format] [-C cfile] [-c param] [-m dev]\n\t\t "
             "[-d number] [-P mailadr] [-v] [-V] [file(s)]\n", name);
-    printf ("\t-f:      Specify the output format. Allowed types are:\n");
+    printf ("\t-f       Specify the output format. Allowed types are:\n");
     printf ("\t         HYLAFAX (default), TIFF, SFF, G3\n");
-    printf ("\t-C:      Name and path of the config file\n");
-    printf ("\t-c:      One Entry from the config-file of type: {value}:{data}\n");
-    printf ("\t-m:      Virtual device name only needed for Hylafax. Default: %s\n", DEFAULT_DEVICE_NAME);
-    printf ("\t-P:      Fax Polling. At the end pollrcvd with \"mailadr\" is called\n");
+    printf ("\t-C       Name and path of the config file\n");
+    printf ("\t-c       One Entry from the config-file of type: {value}:{data}\n");
+    printf ("\t-m       Virtual device name only needed for Hylafax. Default: %s\n", DEFAULT_DEVICE_NAME);
+    printf ("\t-P       Fax Polling. At the end pollrcvd with \"mailadr\" is called\n");
+    printf ("\t-L       Activate Logging (Filename have to be specified in the config file).\n");
     printf ("\t-v       Start in verbose mode\n");
     printf ("\t-V       Show version information\n");
     printf ("\tfile(s): Depending on the format(-f) switch.\n");
@@ -208,7 +390,6 @@ void usage (char *prgName) {
 \*===========================================================================*/
 
 int main (int argc, char **argv) {
-    char *statustext;
     DebugFileOpen();
     dhead ("CFaxSend-main", DCON_CFaxSend);
     CFaxSend fs (FinishFunc);
@@ -220,21 +401,15 @@ int main (int argc, char **argv) {
 
     CDynamicString TelNumber;
     CMultiString   FaxFileList;
-    tUInt          Resolution = 0;
-    tUInt          JobID      = 0;
-    CDynamicString JobTag;
-    CDynamicString MailAddr;
     CDynamicString PollStr;
-    CDynamicString TagLineFormat;
+    FaxFormatType  format     = FaxFormat_Hylafax;
+    tUInt          Resolution = 0;
+    char          *deviceName = DEFAULT_DEVICE_NAME;
+    char          *configName = 0;
+    int            ch;
 
     // find device name
-    FaxFormatType format = FaxFormat_Hylafax;
-    char  HylafaxConfigName[MAX_STRING_SIZE];
-    char *configName = 0;
-    char *deviceName = DEFAULT_DEVICE_NAME;
-    tUInt commID     = 0;
-    int   ch;
-    while ((ch = getopt (argc, argv, "Hh?Vvlpnf:C:c:d:m:P:")) != -1) {
+    while ((ch = getopt (argc, argv, "Hh?VvLlpnf:C:c:d:m:P:")) != -1) {
         switch (ch) {
         default:
             usage (argv[0]);
@@ -242,6 +417,7 @@ int main (int argc, char **argv) {
         case 'V':
             printf ("c2faxsend Version: %s (c) AVM GmbH\n", VERSION);
             return 13;
+        case 'L':
         case 'l':
         case 'p':
         case 'v':
@@ -291,104 +467,54 @@ int main (int argc, char **argv) {
         }
     }
     if (!configName) {
-        if (format == FaxFormat_Hylafax) {
-            sprintf (HylafaxConfigName, HYLAFAX_CONFIG_PREFIX "%s", deviceName);
-            configName = HylafaxConfigName;
-        } else {
-            configName = DEFAULT_CONFIG_FILE;
-        }
+        configName = DEFAULT_CONFIG_FILE;
     }
     if ((optind >= argc) && (PollStr.IsEmpty() == vTrue)) {
         printf ("No fax file specified!\n");
         return 1;
     }
-    if (fs.Config (configName, deviceName, format, argc, argv) == vFalse) {
+    if (fs.Config ((tChar *)configName, deviceName, format, argc, argv) == vFalse) {
         printf ("Initialisation error!\n");
         return 1;
     }
+    sem_init (&syncsema, 0, 0);
     switch (format) {
     case FaxFormat_Hylafax:
         {
-            // ignore -d and -n switch
-            TelNumber.RemoveAll();
-            Resolution = 0;
-
-            CConfigFileParser FileParser;
-            switch (FileParser.ParseFile ((tUChar *)argv[optind])) {
-            case iErr_File_OpenFailed:
-                printf ("Can't open qfile \"%s\".\n", argv[optind]);
-                return 3;
-                break;
-            case i_Okay:
-                break;
-            case iErr_OutOfMemory:
-                dassert (0);
-                printf ("Running out of memory during reading of qfile!\n");
-                return 3;
-                return vFalse;
-            default:
-                dassert (0);
-                printf ("Can't read qfile \"%s\"!\n", argv[optind]);
-                return 3;
-            }
-            CConfigParserSection *mainSect = FileParser.GetMainSection();
-            if (!mainSect) {
-                printf ("Can't find needed information in qfile \"%s\".\n", argv[optind]);
-                return 3;
-            }
-            ConfigAssignment_Item Assignments[] = {
-                { "number",     ConfigAssignment_String,     &TelNumber },
-                { "fax",        ConfigAssignment_StringList, &FaxFileList },
-                { "resolution", ConfigAssignment_UInt,       &Resolution },
-                { "jobid",      ConfigAssignment_UInt,       &JobID },
-                { "jobtag",     ConfigAssignment_String,     &JobTag },
-                { "mailaddr",   ConfigAssignment_String,     &MailAddr },
-                { "poll",       ConfigAssignment_String,     &PollStr },
-                { "tagline",    ConfigAssignment_String,     &TagLineFormat },
-                { 0,            ConfigAssignment_UInt,       0 }
-            };
-            ConfigValuesAssignment (&mainSect->ValueList, &Assignments[0]);
-            // for polling we need the mailaddr
-            if (PollStr.IsEmpty() == vFalse) {
-                PollStr.Set (&MailAddr);
-                if (PollStr.IsEmpty() == vTrue) {
-                    PollStr.Set (" ", 1);
-                }
-            }
-
-            // remove numbers from fax list
-            COneMultiString *pLauf = FaxFileList.GetFirst();
-            while (pLauf != 0) {
-                dassert (pLauf->GetLen() > 0);
-                switch (pLauf->GetChar (0)) {
-                case '0':
-                case '1':
-                case '2':
-                case '3':
-                case '4':
-                case '5':
-                case '6':
-                case '7':
-                case '8':
-                case '9':
-                    {
-                        COneMultiString *toDel = pLauf;
-                        pLauf = pLauf->GetNext();
-                        delete toDel;
+            COneMultiString *pNewStr;
+            char *kommachar;
+            char *tmpqfname;
+            tUInt len;
+            for (; optind < argc; optind++) {
+                tmpqfname = argv[optind];
+                while (tmpqfname) {
+                    kommachar = strchr (tmpqfname, ',');
+                    if (kommachar) {
+                        len = kommachar - tmpqfname;
+                        kommachar++;
+                    } else {
+                        len = s_strlen (tmpqfname);
                     }
-                    break;
-                default:
-                    pLauf = pLauf->GetNext();
-                    break;
+                    if (len) {
+                        pNewStr = new COneMultiString;
+                        if (pNewStr) {
+                            if (pNewStr->Set (tmpqfname, len) == vTrue) {
+                                dprint ("Fax-File=%s ", pNewStr->GetPointer());
+                                FaxFileList.AddLast (pNewStr);
+                            } else {
+                                delete pNewStr;
+                            }
+                        }
+                    }
+                    tmpqfname = kommachar;
                 }
             }
+            pNewStr = FaxFileList.GetFirst();
+            while (pNewStr && ((fs.exitState == 0) || (fs.exitState == 2))) {
+                OneQFileSend (&fs, deviceName, pNewStr);
+                pNewStr = pNewStr->GetNext();
+            }
         }
-
-	// Write "Sending job" into queue file
-	fs.exitState = 255;
-        WriteQFile (argv[optind], commID, &fs, &statustext);
-	fs.exitState = 0;
-
         break;
     case FaxFormat_SFF:
         {
@@ -401,6 +527,7 @@ int main (int argc, char **argv) {
                     delete pNewStr;
                 }
             }
+            OneSendSession (&fs, Resolution, 0, 0, &TelNumber, &PollStr, &FaxFileList);
         }
         break;
     case FaxFormat_TIFF:
@@ -418,73 +545,15 @@ int main (int argc, char **argv) {
                     }
                 }
             }
+            OneSendSession (&fs, Resolution, 0, 0, &TelNumber, &PollStr, &FaxFileList);
         }
         break;
     }
-    if ((FaxFileList.IsEmpty() == vTrue) && (PollStr.IsEmpty() == vTrue)) {
-        printf ("No file to send specified!%s\n\n", (format == FaxFormat_Hylafax) ?
-                " Forgot the format option (-f)?" : "");
-        usage (argv[0]);
-        return 13;
-    }
-    if (TelNumber.IsEmpty() == vFalse) {
-        TelNumber.SetLen (s_strkeep (TelNumber.GetPointer(), (tChar *)"+0123456789*#RrMm"));
-    }
-    if (TelNumber.IsEmpty() == vTrue) {
-        printf ("No telephone number specified!\n\n");
-        usage (argv[0]);
-        return 13;
-    }
-
-    sem_init (&syncsema, 0, 0);
-    if (fs.Send (&TelNumber,
-                 &FaxFileList,
-                 Resolution,
-                 JobID,
-                 &commID,
-                 &PollStr,
-                 &TagLineFormat) == i_Okay) {
-        sem_wait (&syncsema);
-    }
     sem_destroy (&syncsema);
-    dprint ("fs.exitState=%x ", fs.exitState);
-    if (format == FaxFormat_Hylafax) {
-        WriteQFile (argv[optind], commID, &fs, &statustext);
-        // map error codes from mgetty+sendfax to hylafax
-        switch (fs.exitState) {
-        case 2:
-        case 4:
-        case 5:
-        case 6:
-        case 7:
-        case 10:
-        case 12:
-            // retry
-            fs.exitState = 0;
-            break;
-        case 15:
-        default:
-            // error
-            fs.exitState = 1;
-            break;
-        case 0:
-            // okay
-            fs.exitState = 2;
-            break;
-        case 1:
-        case 3:
-        case 11:
-            // reformat
-            fs.exitState = 3;
-            break;
-        }
-        fs.WriteXferLog (PollStr.IsEmpty() ? "SEND" : "POLL", commID, JobTag.GetPointer(), &JobID,
-                         deviceName, &MailAddr, &TelNumber, statustext);
-        dprint ("fs.exitState=%x ", fs.exitState);
-        fs.WriteLog (LOG_INFO, "SESSION END\n");
-    }
-    fs.CloseJobFile();
     DebugFileClose();
+    if (fs.exitState == 13) {
+        usage (argv[0]);
+    }
     return fs.exitState;
 }
 
